
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include <nuon/bios.h>
#include <nuon/mutil.h>
#include <m3dl/pixel.h>
#include <m3dl/dma.h>
#include <m3dl/mdtypes.h>

#include <m3dl/m3dl.h>
#include <nuon/joystick.h>
#include <nuon/video.h>
#include <nuon/mutil.h>
#include <nuon/audio.h>
#include <nuon/nise.h>
#include <nuon/mediaio.h>
#include <nuon/time.h>
#include <nuon/mml2d.h>


#include "config.h"
#include "platform.h"
#include "atari.h"




static int consol;

extern int TRIG_auto[4];	/* autofire */

extern double deltatime;

int Atari_Keyboard(void)
{
  int keyReturn = AKEY_NONE;
  
  if (_Controller[1].status)
  {
    if(ButtonR(_Controller[1]) && ButtonL(_Controller[1]) )
    {
      keyReturn = AKEY_COLDSTART;
    }
  }
  return keyReturn;
}

#define MAX_CLR (256)

mdDRAWCONTEXT        dcx[2];				//Draw Context Structures
static mdBYTE        matrixStack[1024];
static mdTILE        cleartile;
static mdIMAGEDATA   imageData;
static mdBITMAP      bitmap;
static mdIMAGE	     background;	
static uint16        *CurrentPaletteBase;
static uint16        *CurrentPalette;

static unsigned char *DecompressBufferBase;
static unsigned char *DecompressBuffer;
extern int           colortable[256];


void Atari_Initialise(int *argc, char *argv[])
{
    mdUINT32 sdramlen, sdramaddr, i;
    mdUINT32 w = ATARI_WIDTH, h = ATARI_HEIGHT;
    int DecompressBufferSize;

    //Clear ALL Draw areas of drawcontext (DO not execute on a per frame basis!)
    mdCOLOR clearcolor;
    uint16 r,g,b;


    sdramaddr = 0x40000000;

    sdramlen = mdSetBufGRB16B_NOZ_YCC32B(dcx, (mdBYTE*)(sdramaddr), w+48, h+16, 24, 8, w, h);

    VidSetup((mdBYTE*)(dcx[1].buf[dcx[1].actbuf].sdramaddr),dcx[1].buf[dcx[1].actbuf].dmaflags, dcx[1].dispw, dcx[1].disph, VID_VFILTER_2TAP);


    //Set Viewing Frustrum
    // set FOV to 3DS 45.507 and 
    // subrtact 76 from the height to reserve the UI portion on the bottom
    // the 3DS aspect ratio is 1.786 (0x1c937 in 16.16)
    mdSetFrustum((0x2D8000)/360, w, h-76, (0x1C937), 16<<16, 5000<<16);  //(11<<16)/8
    mdSetFogNearFar((512<<16),(5000<<16));

    //Syncronize with VBlank
    _VidSync(1);

    //Setup MPR
    mdSetupMPRChain(2,1);        //1,2

    mdSetRGB(&clearcolor,0,0,0);
    mdClearDraw(dcx, &clearcolor);

    //Clear ALL Display areas of drawcontext (DO not execute on a per frame basis!)
    mdSetRGB(&clearcolor,0,0,0);
    mdClearDisp(dcx, &clearcolor);

    //Setup Clear Screen Sprite (Only Rendering Window)
    mdSetRGB(&cleartile.color,0,100,200);  //100,200);
    mdSetScrRECT(&cleartile.sr,dcx[0].rendx<<4,dcx[0].rendy<<4,mdGetFarZ(),dcx[0].rendw<<4,dcx[0].rendh<<4);

    //Wait for MPR activity to finish
    mdDrawSync();								//Wait for ClearDraw & ClearDisp
    mdSetMatrixStack(matrixStack);


    DecompressBufferSize = (ATARI_WIDTH) * (long) ((ATARI_HEIGHT)+1);
	DecompressBufferBase = (unsigned char *) malloc(DecompressBufferSize+16);
    DecompressBuffer = (unsigned char *) (((long) DecompressBufferBase + 16) & 0xFFFFFFF0);
    CurrentPaletteBase = (unsigned short *) malloc((256*4)+256);
    CurrentPalette = (unsigned short *) (((long) CurrentPaletteBase+256) & 0xFFFFFF00);

    //UV setup (no tiling, no flipping)
	background.u0	= 0<<10;
	background.v0	= 0<<10;
	background.uofs	= 1<<10;
	background.vofs	= 1<<10;

	bitmap.bitmap   = (mdUINT32) DecompressBuffer | 0xA0000000;
    bitmap.clut     = (mdUINT32) CurrentPalette | 0x30000000;  //0x26000000;  //0xA6000002;

	imageData.pixtype       = 0x3;      //0x23 for transparent color 0;
	imageData.miplevels     = 1;
	imageData.width         = ATARI_WIDTH>>2;
	imageData.height        = ATARI_HEIGHT>>2;
	imageData.bmnfo         = &bitmap;

	mdSetScrRECT(&background.sr,0,0,10<<16,(ATARI_WIDTH)<<4,(ATARI_HEIGHT)<<4);
    mdSetRGBA(&background.color, 0, 0, 0, 0);


    // init the drawing palette
  for (i = 0; i < MAX_CLR; i++)
    {
        r = (colortable[i] >> 16) & 0xff;
        g = (colortable[i] >> 8) & 0xff;
        b = (colortable[i]) & 0xff;
        
        CurrentPalette[i] =  
                            ((uint16)(b >> 3)) +
                            ((uint16)(r >> 3) << 5) +
                            ((uint16)(g >> 2) <<10);
    }

}

int Atari_Exit(int run_monitor)
{
  return 0;
}

void Atari_DisplayScreen(UBYTE * ascreen)
{

	//mdCOLOR			dpq;						//Depth Cue Color

	mdDrawSync();				//Wait for Draw Execution
//	_VidSync(0);

	// Swap Display Buffer
	SwapDrawBufGRB(dcx);

	//Set MPR Screen Buffer
	mdActiveDrawContext(&dcx[0]);

 	bitmap.bitmap   = (mdUINT32) ascreen | 0xA0000000;
//    memcpy(DecompressBuffer, ascreen, (ATARI_WIDTH * ATARI_HEIGHT));

//	mdDrawSync();				//Wait for Draw Execution
    mdDrawImage(mpIMG, &background.sr, &background.color, &imageData, (mdUINT32 *) &background.u0);


//printf("Frame\n");

#if 0
{
FILE *out;

out = fopen("out", "wb");
fwrite(colortable, 4, 256, out);
fwrite(ascreen, ATARI_HEIGHT, ATARI_WIDTH, out);
fclose(out);
while (1);
}

#endif
}

#define ATARI_JOYMASK_UP    (~1)
#define ATARI_JOYMASK_DOWN  (~2)
#define ATARI_JOYMASK_LEFT  (~4)
#define ATARI_JOYMASK_RIGHT (~8)

int Atari_PORT(int num)
{    
    int joyReturnLo = 0xf;
    int joyReturnHi = 0xf;
    int nuonJoystick = 1;

    if(num == 1)
        nuonJoystick = 3;
    
    if (_Controller[nuonJoystick].status)
    {
        if(ButtonUp(_Controller[nuonJoystick]))
            joyReturnLo &= ATARI_JOYMASK_UP;

        if(ButtonDown(_Controller[nuonJoystick]))
            joyReturnLo &= ATARI_JOYMASK_DOWN;

        if(ButtonLeft(_Controller[nuonJoystick]))
            joyReturnLo &= ATARI_JOYMASK_LEFT;

        if(ButtonRight(_Controller[nuonJoystick]))
            joyReturnLo &= ATARI_JOYMASK_RIGHT;
    }      

    if (_Controller[nuonJoystick+1].status)
    {
        if(ButtonUp(_Controller[nuonJoystick+1]))
            joyReturnHi &= ATARI_JOYMASK_UP;

        if(ButtonDown(_Controller[nuonJoystick+1]))
            joyReturnHi &= ATARI_JOYMASK_DOWN;

        if(ButtonLeft(_Controller[nuonJoystick+1]))
            joyReturnHi &= ATARI_JOYMASK_LEFT;

        if(ButtonRight(_Controller[nuonJoystick+1]))
            joyReturnHi &= ATARI_JOYMASK_RIGHT;
    }      

    return (joyReturnHi << 4) | (joyReturnLo);                                     
}


int Atari_TRIG(int num)
{
    int trig = 0;
    int i;

    for(i=0; i<4; i++)
    {
        if (_Controller[i+1].status)
        {
            if(ButtonA(_Controller[i+1]))
                trig &= ~(1<<i);
        }
    }

    return trig;
}

int Atari_POT(int num)
{
  return 228;
}

int Atari_CONSOL(void)
{
    // turn all console keys off
    consol = 7;

    if (_Controller[1].status)
    {
        if(ButtonStart(_Controller[1]) )
            consol &= 0x06;                   /* START key ON */

        if(ButtonCLeft(_Controller[1]) )
            consol &= 0x03;                   /* OPTION key ON */

        if(ButtonCRight(_Controller[1]) )
            consol &= 0x05;                   /* SELECT key ON */
    }

  return consol;
}

int Atari_PEN(int vertical)
{
  return vertical ? 0xff : 0;
}

#if 0
void usleep(unsigned long msecs)
{
  //_TimeToSleep(msecs);
}
#endif

/*
$Log: atari_win32.c,v $
Revision 1.3  2001/04/08 05:51:44  knik
sound calls update

Revision 1.2  2001/03/24 10:15:13  knik
included missing header

Revision 1.1  2001/03/18 07:56:48  knik
win32 port

*/
